# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License

from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='hack-assembler',
    version='0.3.1',
    author="itd27m01",
    author_email="igortiunov@gmail.com",
    description="nand2tetris hack assembler",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ITD27M01/hack-assembler",
    packages=['assembler'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    entry_points='''
        [console_scripts]
        assembler=assembler.cli:main
    '''
)
